from serial import Serial

###############################################################################
##                     Recherche des ports UART libre                        ##
###############################################################################
def serialPorts():
    #liste vide pour recevoir les ports libres
    result = []
    #on teste chaque port successivement
    for i in range(2,256):
        port = "COM" + str(i)
        try:
            #essaye de se connecter sur le port
            s = Serial(port)
            s.close()
            #si connection ok, on ajoute le port a la liste des ports libres
            result.append(port)
        except:
            #en cas d'erreur de connection, on passe au port suivant
            pass
    return result

###############################################################################
##                    Configuration de la liaison UART                       ##
###############################################################################
def connectionGPS_V0():
    
    ports = serialPorts()
    
    print(ports)
    
    port = ports[0]
    
    s = Serial(port,9600)
      
    print("UART opérationnel sur", s.portstr, '\n')
    
    return(s)

###############################################################################
##                    Configuration de la liaison UART                       ##
###############################################################################
def connectionGPS_V1():
    ports = serialPorts()
    #si un seul port dispo, c'est parti
    if len(ports) == 1:
            port = ports[0]
    else:
            #sinon, il faut demander
            port = input("Saisissez un autre port " + str(ports) + " : ")
    
    while 1:
            print ('Essai de connection sur le port', port, "... ",end="")
            try:
                    #creation de l'objet UART avec un timeout de 2s (indispensable, sinon risque de blocage)
                    s = Serial(port,9600, timeout=2)
                    print("UART opérationnel sur", s.portstr, '\n')
                    return(s)
            except:
                    print("échec !")
                    ports = serialPorts()
                    port = input("Saisissez un autre port", str(ports), ":")
                    
###############################################################################
##                  Récupération des trames GPS en liste                     ##
###############################################################################
def NmeaToList(s):
    
    bytesRecu = s.readline()      #lit jusqu'au fin de ligne
    
    try:
        #decodage en unicode
        #placé dans un try car cette instruction plante si la trame recue est vide
        textRecu = bytesRecu.decode('utf8')
    except:
        textRecu = ""

    return(textRecu.split(','))

###############################################################################
##                         Extracion des données                             ##
###############################################################################
def extraireHeure(strHeure):
    try:
        #strHeure est de la forme "HHMMSS.00"
        #2 premiers caractere vers heure
        heure =  strHeure[:2]
        #2 suivants vers minute
        minute =  strHeure[2:4]
        #2 suivants vers minute
        seconde =  strHeure[4:6]
        
        #retourne un str mis en forme HH:MM:SS
        return(heure + "h" + minute + "m" + seconde + "s")
        
    except:
        return("non dispo")

def extraireDate(strDate):
    try:
        #strDate est de la forme "JJMMAA"
        #2 premiers caractere vers jour
        jour =  strDate[:2]
        #2 suivants vers mois
        mois =  strDate[2:4]
        #2 suivants vers minute
        annee =  "20"+strDate[4:6]
        
        #retourne un str mis en forme HH:MM:SS
        return(jour + "/" + mois + "/" + annee)
        
    except:
        return("non dispo")
        
def extraireLat(strLatLong):
    try:
        
        return(float(strLatLong[:2]) + float(strLatLong[2:])/60)
        
    except:
        return(0)   
        
def extraireLong(strLatLong):
    try:
        
        return(float(strLatLong[:3]) + float(strLatLong[3:])/60)
        
    except:
        return(0)  

###############################################################################
##                            Corp du programme                              ##
###############################################################################

ser = connectionGPS_V0()

while 1:

    listGPS = NmeaToList(ser)

    #détection d'une trame type $GPRMC qui contient les infos souhaitées
    if listGPS[0] == "$GPRMC":

        #print(listGPS)

        #affichage de l'heure contenu sous forme HHMMSS.00 dans la case 1 de la liste
        print("Heure UTC",extraireHeure(listGPS[1]),"le",extraireDate(listGPS[9]))
        

        #affichage de la latitude
        print("Latitude : ",round(extraireLat(listGPS[3]),4),"° ",listGPS[4])

        #affichage de la longitude
        print("Longitude : ",round(extraireLong(listGPS[5]),4),"° ",listGPS[6])

        print("")



